@@ -21,7 +21,7 @@ def pea(args):
 
 def gateway(args):
     """Start a Gateway Pod"""
-    from jina.peapods.pod import GatewayPod
+    from jina.peapods.pods.gateway import GatewayPod
     with GatewayPod(args) as fs:
         fs.join()
 
@@ -45,8 +45,8 @@ def ping(args):
 
 def client(args):
     """Start a client connects to the gateway"""
-    from jina.clients.python import PyClient
-    PyClient(args)
+    from jina.clients import Client
+    Client(args)
 
 
 def export_api(args):
