@@ -42,28 +42,28 @@ ac_table = {
                 '--socket-out', '--port-ctrl', '--ctrl-with-ipc', '--timeout', '--timeout-ctrl', '--timeout-ready',
                 '--dump-interval', '--exit-no-dump', '--read-only', '--separated-workspace', '--pea-id', '--num-part',
                 '--role', '--skip-on-error', '--memory-hwm', '--runtime', '--max-idle-time', '--daemon', '--log-config',
-                '--log-remote', '--ssh-server', '--ssh-keyfile', '--ssh-password', '--host', '--port-expose',
-                '--port-grpc', '--max-message-size', '--proxy', '--parallel', '--shards', '--polling', '--scheduling',
-                '--uses-before', '--uses-after', '--shutdown-idle'],
-        'flow': ['--help', '--uses', '--logserver', '--logserver-config', '--optimize-level', '--output-type',
-                 '--output-path', '--inspect'],
+                '--log-remote', '--log-id', '--ssh-server', '--ssh-keyfile', '--ssh-password', '--host',
+                '--port-expose', '--port-grpc', '--max-message-size', '--proxy', '--remote-access', '--parallel',
+                '--shards', '--polling', '--scheduling', '--uses-before', '--uses-after', '--shutdown-idle'],
+        'flow': ['--help', '--uses', '--logserver', '--logserver-config', '--log-id', '--optimize-level',
+                 '--output-type', '--output-path', '--inspect'],
         'gateway': ['--help', '--name', '--identity', '--uses', '--py-modules', '--uses-internal', '--entrypoint',
                     '--pull-latest', '--volumes', '--port-in', '--port-out', '--host-in', '--host-out', '--socket-in',
                     '--socket-out', '--port-ctrl', '--ctrl-with-ipc', '--timeout', '--timeout-ctrl', '--timeout-ready',
                     '--dump-interval', '--exit-no-dump', '--read-only', '--separated-workspace', '--pea-id',
                     '--num-part', '--role', '--skip-on-error', '--memory-hwm', '--runtime', '--max-idle-time',
-                    '--daemon', '--log-config', '--log-remote', '--ssh-server', '--ssh-keyfile', '--ssh-password',
-                    '--host', '--port-expose', '--port-grpc', '--max-message-size', '--proxy', '--prefetch',
-                    '--prefetch-on-recv', '--allow-spawn', '--rest-api', '--check-version', '--compress',
-                    '--compress-hwm', '--compress-lwm'],
+                    '--daemon', '--log-config', '--log-remote', '--log-id', '--ssh-server', '--ssh-keyfile',
+                    '--ssh-password', '--host', '--port-expose', '--port-grpc', '--max-message-size', '--proxy',
+                    '--remote-access', '--prefetch', '--prefetch-on-recv', '--rest-api', '--check-version',
+                    '--compress', '--compress-hwm', '--compress-lwm'],
         'ping': ['--help', '--timeout', '--retries', '--print-response'],
         'check': ['--help', '--summary-exec', '--summary-driver'], 'hub login': ['--help'],
         'hub new': ['--help', '--output-dir', '--template', '--type', '--overwrite'],
         'hub init': ['--help', '--output-dir', '--template', '--type', '--overwrite'],
         'hub create': ['--help', '--output-dir', '--template', '--type', '--overwrite'],
         'hub build': ['--help', '--username', '--password', '--registry', '--repository', '--pull', '--push',
-                      '--dry-run', '--prune-images', '--raise-error', '--test-uses', '--host-info', '--daemon'],
-        'hub push': ['--help', '--username', '--password', '--registry', '--repository'],
+                      '--dry-run', '--prune-images', '--raise-error', '--test-uses', '--test-level', '--host-info',
+                      '--daemon'], 'hub push': ['--help', '--username', '--password', '--registry', '--repository'],
         'hub pull': ['--help', '--username', '--password', '--registry', '--repository'],
         'hub list': ['--help', '--name', '--kind', '--keywords', '--type', '--local-only'],
         'hub': ['--help', 'login', 'new', 'init', 'create', 'build', 'push', 'pull', 'list'],
@@ -72,8 +72,10 @@ ac_table = {
                 '--socket-out', '--port-ctrl', '--ctrl-with-ipc', '--timeout', '--timeout-ctrl', '--timeout-ready',
                 '--dump-interval', '--exit-no-dump', '--read-only', '--separated-workspace', '--pea-id', '--num-part',
                 '--role', '--skip-on-error', '--memory-hwm', '--runtime', '--max-idle-time', '--daemon', '--log-config',
-                '--log-remote', '--ssh-server', '--ssh-keyfile', '--ssh-password', '--host', '--port-expose',
-                '--port-grpc', '--max-message-size', '--proxy'], 'log': ['--help', '--groupby-regex', '--refresh-time'],
-        'client': ['--help', '--host', '--port-expose', '--port-grpc', '--max-message-size', '--proxy', '--batch-size',
-                   '--mode', '--top-k', '--mime-type', '--callback-on-body', '--timeout-ready', '--skip-dry-run'],
+                '--log-remote', '--log-id', '--ssh-server', '--ssh-keyfile', '--ssh-password', '--host',
+                '--port-expose', '--port-grpc', '--max-message-size', '--proxy', '--remote-access'],
+        'log': ['--help', '--groupby-regex', '--refresh-time'],
+        'client': ['--help', '--host', '--port-expose', '--port-grpc', '--max-message-size', '--proxy',
+                   '--remote-access', '--batch-size', '--mode', '--top-k', '--mime-type', '--callback-on',
+                   '--timeout-ready', '--skip-dry-run', '--continue-on-error'],
         'export-api': ['--help', '--yaml-path', '--json-path']}}
