@@ -3,8 +3,9 @@ __license__ = "Apache-2.0"
 
 import os
 
-from . import __jina_env__, import_classes
-from .helper import colored, print_dep_tree_rst
+from . import __jina_env__
+from .helper import colored
+from .importer import import_classes, _print_dep_tree_rst
 from .logging import default_logger
 
 if False:
@@ -22,7 +23,7 @@ class ImportChecker:
 
         if args.summary_exec:
             with open(args.summary_exec, 'w') as fp:
-                print_dep_tree_rst(fp, _r, 'Executor')
+                _print_dep_tree_rst(fp, _r, 'Executor')
 
         default_logger.info('\navailable hub executors\n'.upper())
 
@@ -30,19 +31,19 @@ class ImportChecker:
 
         if args.summary_exec:
             with open(args.summary_exec, 'w') as fp:
-                print_dep_tree_rst(fp, _r, 'Executor')
+                _print_dep_tree_rst(fp, _r, 'Executor')
 
         default_logger.info('\navailable drivers\n'.upper())
         _r = import_classes('jina.drivers', show_import_table=True, import_once=False)
 
         if args.summary_driver:
             with open(args.summary_driver, 'w') as fp:
-                print_dep_tree_rst(fp, _r, 'Driver')
+                _print_dep_tree_rst(fp, _r, 'Driver')
 
         # check available driver group
 
         default_logger.info('\nenvironment variables\n'.upper())
-        default_logger.info('\n'.join('%-20s\t%s' % (k, os.environ.get(k, colored('(unset)', 'yellow'))) for k in
+        default_logger.info('\n'.join(f'{k:<20}\t{os.environ.get(k, colored("(unset)", "yellow"))}' for k in
                                       __jina_env__))
 
 
@@ -50,18 +51,17 @@ class NetworkChecker:
     """Check if a BasePod is running or not """
 
     def __init__(self, args: 'argparse.Namespace'):
-        from jina.peapods.pea import send_ctrl_message
-        from jina.proto import jina_pb2
-        from jina.logging.profile import TimeContext
+        from .peapods.peas import send_ctrl_message
+        from .logging.profile import TimeContext
         from google.protobuf.json_format import MessageToJson
         import time
-        ctrl_addr = 'tcp://%s:%d' % (args.host, args.port)
+        ctrl_addr = f'tcp://{args.host}:{args.port}'
         try:
             total_time = 0
             total_success = 0
             for j in range(args.retries):
-                with TimeContext('ping %s at %d round' % (ctrl_addr, j), default_logger) as tc:
-                    r = send_ctrl_message(ctrl_addr, jina_pb2.Request.ControlRequest.STATUS, timeout=args.timeout)
+                with TimeContext(f'ping {ctrl_addr} at {j} round', default_logger) as tc:
+                    r = send_ctrl_message(ctrl_addr, 'STATUS', timeout=args.timeout)
                     if not r:
                         default_logger.warning('not responding, retry (%d/%d) in 1s' % (j + 1, args.retries))
                     else:
