@@ -6,7 +6,6 @@ import pickle
 import re
 import subprocess
 import tempfile
-import uuid
 from datetime import datetime
 from pathlib import Path
 from types import SimpleNamespace
@@ -15,16 +14,17 @@ from typing import Dict, Any, Union, TypeVar, Type, TextIO, List
 import ruamel.yaml.constructor
 from ruamel.yaml import StringIO
 
-from .decorators import as_train_method, as_update_method, store_init_kwargs, as_aggregate_method
+from .decorators import as_train_method, as_update_method, store_init_kwargs, as_aggregate_method, wrap_func
 from .metas import get_default_metas, fill_metas_with_defaults
 from ..excepts import EmptyExecutorYAML, BadWorkspace, BadPersistantFile, NoDriverForRequest, UnattachedDriver
-from ..helper import yaml, PathImporter, expand_dict, expand_env_var, get_local_config_source
+from ..helper import yaml, expand_dict, expand_env_var, get_local_config_source, typename, get_random_identity
+from ..importer import PathImporter
 from ..logging import JinaLogger
 from ..logging.profile import TimeContext
 
 if False:
+    from ..peapods.peas import BasePea
     from ..drivers import BaseDriver
-    from ..peapods.pea import BasePea
 
 __all__ = ['BaseExecutor', 'AnyExecutor', 'ExecutorType']
 
@@ -33,7 +33,7 @@ AnyExecutor = TypeVar('AnyExecutor', bound='BaseExecutor')
 # some variables may be self-referred and they must be resolved at here
 _ref_desolve_map = SimpleNamespace()
 _ref_desolve_map.__dict__['metas'] = SimpleNamespace()
-_ref_desolve_map.__dict__['metas'].__dict__['replica_id'] = 0
+_ref_desolve_map.__dict__['metas'].__dict__['pea_id'] = 0
 _ref_desolve_map.__dict__['metas'].__dict__['separated_workspace'] = False
 
 
@@ -61,25 +61,17 @@ class ExecutorType(type):
 
     @staticmethod
     def register_class(cls):
-        prof_funcs = ['train', 'encode', 'add', 'query', 'craft', 'score', 'evaluate']
         update_funcs = ['train', 'add']
         train_funcs = ['train']
         aggregate_funcs = ['evaluate']
 
-        def wrap_func(func_lst, wrapper):
-            for f_name in func_lst:
-                if hasattr(cls, f_name):
-                    setattr(cls, f_name, wrapper(getattr(cls, f_name)))
-
         reg_cls_set = getattr(cls, '_registered_class', set())
         if cls.__name__ not in reg_cls_set or getattr(cls, 'force_register', False):
-            cls.__init__ = store_init_kwargs(cls.__init__)
-            # if 'JINA_PROFILING' in os.environ:
-            #     wrap_func(prof_funcs, profiling)
 
-            wrap_func(train_funcs, as_train_method)
-            wrap_func(update_funcs, as_update_method)
-            wrap_func(aggregate_funcs, as_aggregate_method)
+            wrap_func(cls, ['__init__'], store_init_kwargs)
+            wrap_func(cls, train_funcs, as_train_method)
+            wrap_func(cls, update_funcs, as_update_method)
+            wrap_func(cls, aggregate_funcs, as_aggregate_method)
 
             reg_cls_set.add(cls.__name__)
             setattr(cls, '_registered_class', reg_cls_set)
@@ -126,6 +118,7 @@ class BaseExecutor(metaclass=ExecutorType):
 
     """
     store_args_kwargs = False  #: set this to ``True`` to save ``args`` (in a list) and ``kwargs`` (in a map) in YAML config
+    exec_methods = ['encode', 'add', 'query', 'craft', 'score', 'evaluate', 'predict', 'query_by_id']
 
     def __init__(self, *args, **kwargs):
         if isinstance(args, tuple) and len(args) > 0:
@@ -150,7 +143,7 @@ class BaseExecutor(metaclass=ExecutorType):
                 self.on_gpu = False
 
     def _post_init_wrapper(self, _metas: Dict = None, _requests: Dict = None, fill_in_metas: bool = True) -> None:
-        with TimeContext('post initiating, this may take some time', self.logger):
+        with TimeContext('post_init may take some time', self.logger):
             if fill_in_metas:
                 if not _metas:
                     _metas = get_default_metas()
@@ -162,8 +155,8 @@ class BaseExecutor(metaclass=ExecutorType):
                 self._fill_metas(_metas)
                 self._fill_requests(_requests)
 
-            _before = set(list(vars(self).keys()))
             self._check_on_gpu()
+            _before = set(list(vars(self).keys()))
             self.post_init()
             self._post_init_vars = {k for k in vars(self) if k not in _before}
 
@@ -199,13 +192,13 @@ class BaseExecutor(metaclass=ExecutorType):
             elif type(getattr(self, k)) == type(v):
                 setattr(self, k, v)
         if not getattr(self, 'name', None):
-            _id = str(uuid.uuid4()).split('-')[0]
-            _name = f'{self.__class__.__name__}-{_id}'
-            if self.warn_unnamed:
+            _id = get_random_identity().split('-')[0]
+            _name = f'{typename(self)}-{_id}'
+            if getattr(self, 'warn_unnamed', False):
                 self.logger.warning(
-                    'this executor is not named, i will call it "%s". '
+                    f'this executor is not named, i will call it "{_name}". '
                     'naming is important as it provides an unique identifier when '
-                    'persisting this executor on disk.' % _name)
+                    'persisting this executor on disk.')
             setattr(self, 'name', _name)
         if unresolved_attr:
             _tmp = vars(self)
@@ -270,9 +263,9 @@ class BaseExecutor(metaclass=ExecutorType):
         """ Get the path of the current workspace.
 
         :return: if ``separated_workspace`` is set to ``False`` then ``metas.workspace`` is returned,
-                otherwise the ``metas.replica_workspace`` is returned
+                otherwise the ``metas.pea_workspace`` is returned
         """
-        work_dir = self.replica_workspace if self.separated_workspace else self.workspace  # type: str
+        work_dir = self.pea_workspace if self.separated_workspace else self.workspace  # type: str
         return work_dir
 
     def get_file_from_workspace(self, name: str) -> str:
@@ -280,7 +273,7 @@ class BaseExecutor(metaclass=ExecutorType):
 
         :param name: the name of the file
 
-        :return depending on ``metas.separated_workspace`` the file could be located in ``metas.workspace`` or ``metas.replica_workspace``
+        :return depending on ``metas.separated_workspace`` the file could be located in ``metas.workspace`` or ``metas.pea_workspace``
         """
         Path(self.current_workspace).mkdir(parents=True, exist_ok=True)
         return os.path.join(self.current_workspace, name)
@@ -306,9 +299,10 @@ class BaseExecutor(metaclass=ExecutorType):
         self.logger = JinaLogger(self.__class__.__name__)
         try:
             self._post_init_wrapper(fill_in_metas=False)
-        except ImportError as ex:
-            self.logger.warning('ImportError is often caused by a missing component, '
-                                'which often can be solved by "pip install" relevant package. %s' % ex, exc_info=True)
+        except ModuleNotFoundError as ex:
+            self.logger.warning(f'{typename(ex)} is often caused by a missing component, '
+                                f'which often can be solved by "pip install" relevant package: {repr(ex)}',
+                                exc_info=True)
 
     def train(self, *args, **kwargs) -> None:
         """
@@ -320,9 +314,9 @@ class BaseExecutor(metaclass=ExecutorType):
         """Touch the executor and change ``is_updated`` to ``True`` so that one can call :func:`save`. """
         self.is_updated = True
 
-    def save(self, filename: str = None) -> bool:
+    def save(self, filename: str = None):
         """
-        Persist data of this executor to the :attr:`workspace` (or :attr:`replica_workspace`). The data could be
+        Persist data of this executor to the :attr:`workspace` (or :attr:`pea_workspace`). The data could be
         a file or collection of files produced/used during an executor run.
 
         These are some of the common data that you might want to persist:
@@ -341,31 +335,28 @@ class BaseExecutor(metaclass=ExecutorType):
         :param filename: file path of the serialized file, if not given then :attr:`save_abspath` is used
         :return: successfully persisted or not
         """
-        if not self.is_updated:
-            self.logger.info(f'no update since {self._last_snapshot_ts:%Y-%m-%d %H:%M:%S%z}, will not save. '
+        if not self.read_only and self.is_updated:
+            f = filename or self.save_abspath
+            if not f:
+                f = tempfile.NamedTemporaryFile('w', delete=False, dir=os.environ.get('JINA_EXECUTOR_WORKDIR', None)).name
+
+            if self.max_snapshot > 0 and os.path.exists(f):
+                bak_f = f + f'.snapshot-{self._last_snapshot_ts.strftime("%Y%m%d%H%M%S") or "NA"}'
+                os.rename(f, bak_f)
+                self._snapshot_files.append(bak_f)
+                if len(self._snapshot_files) > self.max_snapshot:
+                    d_f = self._snapshot_files.pop(0)
+                    if os.path.exists(d_f):
+                        os.remove(d_f)
+            with open(f, 'wb') as fp:
+                pickle.dump(self, fp)
+                self._last_snapshot_ts = datetime.now()
+            self.is_updated = False
+            self.logger.success(f'artifacts of this executor ({self.name}) is persisted to {f}')
+        else:
+            if not self.is_updated:
+                self.logger.info(f'no update since {self._last_snapshot_ts:%Y-%m-%d %H:%M:%S%z}, will not save. '
                              'If you really want to save it, call "touch()" before "save()" to force saving')
-            return False
-
-        self.is_updated = False
-        f = filename or self.save_abspath
-        if not f:
-            f = tempfile.NamedTemporaryFile('w', delete=False, dir=os.environ.get('JINA_EXECUTOR_WORKDIR', None)).name
-
-        if self.max_snapshot > 0 and os.path.exists(f):
-            bak_f = f + '.snapshot-%s' % (self._last_snapshot_ts.strftime('%Y%m%d%H%M%S') or 'NA')
-            os.rename(f, bak_f)
-            self._snapshot_files.append(bak_f)
-            if len(self._snapshot_files) > self.max_snapshot:
-                d_f = self._snapshot_files.pop(0)
-                if os.path.exists(d_f):
-                    os.remove(d_f)
-
-        with open(f, 'wb') as fp:
-            pickle.dump(self, fp)
-            self._last_snapshot_ts = datetime.now()
-
-        self.logger.success(f'artifacts of this executor ({self.name}) is persisted to {f}')
-        return True
 
     def save_config(self, filename: str = None) -> bool:
         """
@@ -380,20 +371,20 @@ class BaseExecutor(metaclass=ExecutorType):
             f = tempfile.NamedTemporaryFile('w', delete=False, dir=os.environ.get('JINA_EXECUTOR_WORKDIR', None)).name
         with open(f, 'w', encoding='utf8') as fp:
             yaml.dump(self, fp)
-        self.logger.info('executor\'s yaml config is save to %s' % f)
+        self.logger.info(f'executor\'s yaml config is save to {f}')
 
         self.is_updated = _updated
         return True
 
     @classmethod
     def load_config(cls: Type[AnyExecutor], source: Union[str, TextIO], separated_workspace: bool = False,
-                    replica_id: int = 0) -> AnyExecutor:
+                    pea_id: int = 0, read_only: bool = False) -> AnyExecutor:
         """Build an executor from a YAML file.
 
-        :param filename: the file path of the YAML file or a ``TextIO`` stream to be loaded from
+        :param source: the file path of the YAML file or a ``TextIO`` stream to be loaded from
         :param separated_workspace: the dump and data files associated to this executor will be stored separately for
-                each replica, which will be indexed by the ``replica_id``
-        :param replica_id: the id of the storage of this replica, only effective when ``separated_workspace=True``
+                each parallel pea, which will be indexed by the ``pea_id``
+        :param pea_id: the id of the storage of this parallel pea, only effective when ``separated_workspace=True``
         :return: an executor object
         """
         if not source: raise FileNotFoundError
@@ -421,7 +412,8 @@ class BaseExecutor(metaclass=ExecutorType):
                         raise TypeError(f'{type(mod)!r} is not acceptable, only str or list are acceptable')
 
                 tmp['metas']['separated_workspace'] = separated_workspace
-                tmp['metas']['replica_id'] = replica_id
+                tmp['metas']['pea_id'] = pea_id
+                tmp['metas']['read_only'] = read_only
 
             else:
                 raise EmptyExecutorYAML(f'{source} is empty? nothing to read from there')
@@ -452,6 +444,7 @@ class BaseExecutor(metaclass=ExecutorType):
         """
         Release the resources as executor is destroyed, need to be overrided
         """
+        self.save()
         self.logger.close()
 
     def __enter__(self):
@@ -524,13 +517,13 @@ class BaseExecutor(metaclass=ExecutorType):
     def _get_dump_path_from_config(meta_config: Dict):
         if 'name' in meta_config:
             if meta_config.get('separated_workspace', False) is True:
-                if 'replica_id' in meta_config and isinstance(meta_config['replica_id'], int):
-                    work_dir = meta_config['replica_workspace']
+                if 'pea_id' in meta_config and isinstance(meta_config['pea_id'], int):
+                    work_dir = meta_config['pea_workspace']
                     dump_path = os.path.join(work_dir, f'{meta_config["name"]}.{"bin"}')
                     if os.path.exists(dump_path):
                         return dump_path
                 else:
-                    raise BadWorkspace('separated_workspace=True but replica_id is unset or set to a bad value')
+                    raise BadWorkspace('separated_workspace=True but pea_id is unset or set to a bad value')
             else:
                 dump_path = os.path.join(meta_config.get('workspace', os.getcwd()),
                                          f'{meta_config["name"]}.{"bin"}')
