@@ -93,7 +93,7 @@ class CompoundExecutor(BaseExecutor):
         c[0].add(obj)
 
     .. note::
-        All components ``workspace`` and ``replica_workspace`` are overrided by their :class:`CompoundExecutor` counterparts.
+        All components ``workspace`` and ``pea_workspace`` are overrided by their :class:`CompoundExecutor` counterparts.
 
     .. warning::
 
@@ -220,7 +220,7 @@ class CompoundExecutor(BaseExecutor):
         for c in self.components:
             c.is_trained = val
 
-    def save(self, filename: str = None) -> bool:
+    def save(self, filename: str = None):
         """
         Serialize this compound executor along with all components in it to binary files
 
@@ -229,11 +229,9 @@ class CompoundExecutor(BaseExecutor):
 
         It uses ``pickle`` for dumping.
         """
-
         for c in self.components:
             c.save()
         super().save(filename=filename)  # do i really need to save the compound executor itself
-        return True
 
     @property
     def components(self) -> List[AnyExecutor]:
@@ -265,7 +263,7 @@ class CompoundExecutor(BaseExecutor):
         for c in self.components:
             c.separated_workspace = self.separated_workspace
             c.workspace = self.workspace
-            c.replica_workspace = self.current_workspace
+            c.pea_workspace = self.current_workspace
 
     def _resolve_routes(self) -> None:
         if self._routes:
@@ -299,15 +297,13 @@ class CompoundExecutor(BaseExecutor):
             raise AttributeError(f'bad names: {comp_name} and {comp_fn_name}')
 
     def _set_routes(self) -> None:
-        import inspect
         # add all existing routes
         r = defaultdict(list)
-        common = {}  # set(dir(BaseExecutor()))
 
         for c in self.components:
-            for m, _ in inspect.getmembers(c, predicate=inspect.ismethod):
-                if not m.startswith('_') and m not in common:
-                    r[m].append((c.name, getattr(c, m)))
+            for method in BaseExecutor.exec_methods:
+                if hasattr(c, method):
+                    r[method].append((c.name, getattr(c, method)))
 
         new_routes = []
         bad_routes = []
@@ -324,8 +320,7 @@ class CompoundExecutor(BaseExecutor):
                     new_routes.append(new_r)
                 else:
                     self.logger.warning(
-                        'function "%s" appears multiple times in %s, it needs to be resolved manually before using.' % (
-                            k, v))
+                        f'function "{k}" appears multiple times in {v}, it needs to be resolved manually before using.')
                     bad_routes.append(k)
         if new_routes:
             self.logger.debug(f'new functions added: {new_routes!r}')
