@@ -3,8 +3,6 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
-from ..logging import default_logger
-
 import inspect
 from functools import wraps
 from typing import Callable, Any, Union, Iterator, List, Optional
@@ -12,7 +10,8 @@ from typing import Callable, Any, Union, Iterator, List, Optional
 import numpy as np
 
 from .metas import get_default_metas
-from ..helper import batch_iterator
+from ..helper import batch_iterator, typename, convert_tuple_to_list
+from ..logging import default_logger
 
 
 def as_aggregate_method(func: Callable) -> Callable:
@@ -52,8 +51,8 @@ def as_train_method(func: Callable) -> Callable:
     @wraps(func)
     def arg_wrapper(self, *args, **kwargs):
         if self.is_trained:
-            self.logger.warning('"%s" has been trained already, '
-                                'training it again will override the previous training' % self.__class__.__name__)
+            self.logger.warning(f'"{typename(self)}" has been trained already, '
+                                'training it again will override the previous training')
         f = func(self, *args, **kwargs)
         self.is_trained = True
         return f
@@ -61,6 +60,29 @@ def as_train_method(func: Callable) -> Callable:
     return arg_wrapper
 
 
+def as_success_fn(func: Callable) -> Callable:
+    @wraps(func)
+    def arg_wrapper(self, *args, **kwargs):
+        f = func(self, *args, **kwargs)
+        self._exec_fn_success = True
+        return f
+
+    return arg_wrapper
+
+
+def wrap_func(cls, func_lst, wrapper):
+    """ Wrapping a class method only once, inherited but not overrided method will not be wrapped again
+
+    :param cls: class
+    :param func_lst: function list to wrap
+    :param wrapper: the wrapper
+    :return:
+    """
+    for f_name in func_lst:
+        if hasattr(cls, f_name) and all(getattr(cls, f_name) != getattr(i, f_name, None) for i in cls.mro()[1:]):
+            setattr(cls, f_name, wrapper(getattr(cls, f_name)))
+
+
 def as_ndarray(func: Callable, dtype=np.float32) -> Callable:
     """Convert an :class:`BaseExecutor` function returns to a ``numpy.ndarray``,
     the following type are supported: `EagerTensor`, `Tensor`, `list`
@@ -127,6 +149,7 @@ def store_init_kwargs(func: Callable) -> Callable:
             self._init_kwargs_dict.update(tmp)
         else:
             self._init_kwargs_dict = tmp
+        convert_tuple_to_list(self._init_kwargs_dict)
         f = func(self, *args, **kwargs)
         return f
 
@@ -221,7 +244,7 @@ def batching(func: Callable[[Any], np.ndarray] = None,
 
             b_size = (batch_size(data) if callable(batch_size) else batch_size) or getattr(args[0], 'batch_size', None)
             # no batching if b_size is None
-            if b_size is None:
+            if b_size is None or data is None:
                 return func(*args, **kwargs)
 
             default_logger.debug(
@@ -314,11 +337,12 @@ def batching_multi_input(func: Callable[[Any], np.ndarray] = None,
     def _batching(func):
         @wraps(func)
         def arg_wrapper(*args, **kwargs):
+            data = args[slice_on]
             # priority: decorator > class_attribute
             # by default data is in args[1:] (self needs to be taken into account)
             b_size = batch_size or getattr(args[0], 'batch_size', None)
             # no batching if b_size is None
-            if b_size is None:
+            if b_size is None or data is None:
                 return func(*args, **kwargs)
 
             args = list(args)
