@@ -1,6 +1,3 @@
-__copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
-__license__ = "Apache-2.0"
-
 """The default meta config that all executors follow, they can be overrided by the YAML config
 
 .. warning::
@@ -74,36 +71,42 @@ Any executor inherited from :class:`BaseExecutor` always has the following **met
         :type: str/List[str]
         :default: ``None``
 
-    .. confval:: replica_id
+    .. confval:: pea_id
 
-        the integer index used for distinguish each replica of this executor, useful in :attr:`replica_workspace`
+        the integer index used for distinguish each parallel pea of this executor, useful in :attr:`pea_workspace`
 
         :type: int
-        :default: ``'{root.metas.replica_id}'``
+        :default: ``'{root.metas.pea_id}'``
 
     .. confval:: separated_workspace
 
-        whether to isolate the data of the parallel of this executor. If ``True``, then each replica works in its own
-        workspace specified in :attr:`replica_workspace`
+        whether to isolate the data of the parallel of this executor. If ``True``, then each parallel pea works in its own
+        workspace specified in :attr:`pea_workspace`
 
         :type: bool
         :default: ``'{root.metas.separated_workspace}'``
         
-    .. confval:: replica_workspace
+    .. confval:: pea_workspace
 
-        the workspace of each replica, useful when :attr:`separated_workspace` is set to True. All data and IO operations
-        related to this replica will be conducted under this workspace. It is often set as the sub-directory of :attr:`workspace`.
+        the workspace of each parallel pea, useful when :attr:`separated_workspace` is set to True. All data and IO operations
+        related to this parallel pea will be conducted under this workspace. It is often set as the sub-directory of :attr:`workspace`.
 
         :type: str
-        :default: ``'{root.metas.workspace}/{root.metas.name}-{root.metas.replica_id}'``
+        :default: ``'{root.metas.workspace}/{root.metas.name}-{root.metas.pea_id}'``
 
+    .. confval:: read_only
+
+        do not allow the pod to modify the model, save calls will be ignored. If set to true no serialization of the executor
+
+        :type: bool
+        :default: ``False``
 
     .. warning::
         ``name`` and ``workspace`` must be set if you want to serialize/deserialize this executor.
 
     .. note::
 
-        ``separated_workspace``, ``replica_workspace`` and ``replica_id`` is set in a way that when the executor ``A`` is used as
+        ``separated_workspace``, ``pea_workspace`` and ``pea_id`` is set in a way that when the executor ``A`` is used as
         a component of a :class:`jina.executors.compound.CompoundExecutor` ``B``, then ``A``'s setting will be overrided by B's counterpart.
 
     These **meta** fields can be accessed via `self.is_trained` or loaded from a YAML config via :func:`load_config`:
@@ -123,6 +126,10 @@ Any executor inherited from :class:`BaseExecutor` always has the following **met
 
 """
 
+__copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
+__license__ = "Apache-2.0"
+
+
 from typing import Dict, Union, List
 
 _defaults = None
