@@ -1,12 +1,11 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
-import json
-from typing import Optional
+from typing import Optional, Iterator, Any
 
 from google.protobuf.json_format import Parse
+from jina import Document
 from jina.executors.indexers.keyvalue import BinaryPbIndexer
-from jina.proto import jina_pb2
 
 
 class LevelDBIndexer(BinaryPbIndexer):
@@ -16,44 +15,54 @@ class LevelDBIndexer(BinaryPbIndexer):
 
     def get_add_handler(self):
         """Get the database handler
-
         """
         import plyvel
         return plyvel.DB(self.index_abspath, create_if_missing=True)
 
     def get_create_handler(self):
         """Get the database handler
-
         """
         import plyvel
         return plyvel.DB(self.index_abspath, create_if_missing=True)
 
-    def add(self, objs):
-        """Add a JSON-friendly object to the indexer
-
-        :param objs: objects can be serialized into JSON format
-        """
-        with self.write_handler.write_batch() as h:
-            for k, obj in objs.items():
-                key = k.encode('utf8')
-                value = json.dumps(obj).encode('utf8')
-                h.put(key, value)
-
     def get_query_handler(self):
         """Get the database handler
-
         """
         import plyvel
         return plyvel.DB(self.index_abspath, create_if_missing=True)
 
-    def query(self, key: str, *args, **kwargs) -> Optional['jina_pb2.Document']:
+    def query(self, key: Any, *args, **kwargs) -> Optional[Any]:
         """Find the protobuf chunk/doc using id
-
         :param key: ``id``
         :return: protobuf chunk or protobuf document
         """
-        v = self.query_handler.get(key.encode('utf8'))
+        v = self.query_handler.get(bytes(key))
         value = None
         if v is not None:
-            value = Parse(json.loads(v.decode('utf8')), jina_pb2.Document())
+            value = Parse(v.decode('utf8'), Document())
         return value
+
+    def add(self, keys: Iterator[int], values: Iterator[bytes], *args, **kwargs):
+        """Add a JSON-friendly object to the indexer
+        :param objs: objects can be serialized into JSON format
+        """
+        with self.query_handler.write_batch() as h:
+            for k, v in zip(keys, values):
+                h.put(bytes(k), v)
+
+    def update(self, keys: Iterator[int], values: Iterator[bytes], *args, **kwargs):
+        missed = []
+        for key in keys:
+            if self.query_handler.get(bytes(key)) is None:
+                missed.append(key)
+        if missed:
+            raise KeyError(f'Key(s) {missed} were not found in {self.save_abspath}')
+
+        self.delete(keys)
+        self.add(keys, values)
+        return
+
+    def delete(self, keys: Iterator[int], *args, **kwargs):
+        with self.query_handler.write_batch() as h:
+            for k in keys:
+                h.delete(bytes(k))
