@@ -1,16 +1,21 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
+import time
+from typing import Optional, Dict
+
 from . import default_logger
-from .queue import __sse_queue__, __profile_queue__
 from .. import JINA_GLOBAL, __version__
-from ..helper import yaml
+from ..importer import ImportExtensions
 
 
-def start_sse_logger(server_config_path: str, flow_yaml: str = None):
+def start_sse_logger(log_config: Dict,
+                     log_id: str,
+                     flow_yaml: Optional[str] = None):
     """Start a logger that emits server-side event from the log queue, so that one can use a browser to monitor the logs
 
-    :param server_config_path: Path to the server configuration file path
+    :param log_config: configuration of the sse server
+    :param log_id: log-id of `Flow`
     :param flow_yaml: Flow yaml description
 
     Example:
@@ -28,68 +33,54 @@ def start_sse_logger(server_config_path: str, flow_yaml: str = None):
         };
 
     """
-    try:
+    with ImportExtensions(required=True):
         from flask import Flask, Response, jsonify
         from flask_cors import CORS
         from gevent.pywsgi import WSGIServer
         import gevent
-    except ImportError:
-        raise ImportError('Flask or its dependencies are not fully installed, '
-                          'they are required for serving HTTP requests.'
-                          'Please use pip install "jina[http]" to install it.')
 
-    try:
-        with open(server_config_path) as fp:
-            _config = yaml.load(fp)
-    except Exception as ex:
-        default_logger.error(ex)
-    JINA_GLOBAL.logserver.address = f'http://{_config["host"]}:{_config["port"]}'
+    JINA_GLOBAL.logserver.address = f'http://{log_config["host"]}:{log_config["port"]}'
 
     JINA_GLOBAL.logserver.ready = JINA_GLOBAL.logserver.address + \
-                                  _config['endpoints']['ready']
+                                  log_config['endpoints']['ready']
     JINA_GLOBAL.logserver.shutdown = JINA_GLOBAL.logserver.address + \
-                                     _config['endpoints']['shutdown']
+                                     log_config['endpoints']['shutdown']
 
     app = Flask(__name__)
     CORS(app)
-    server = WSGIServer((_config['host'], _config['port']), app, log=None)
+    server = WSGIServer((log_config['host'], log_config['port']), app, log=None)
 
-    def _log_stream():
-        while True:
-            try:
-                gevent.sleep(0)
-                message = __sse_queue__.get()
-                yield f'data: {message.msg}\n\n'
-            except EOFError:
-                yield 'LOG ENDS\n\n'
-                break
+    def _log_stream(path):
+        import glob
+        # fluentd creates files under this path with some tag based on day, so as temp solution,
+        # just get the first file matching this patter once it appears
+        file = f'{path}/{log_id}/log.log'
+        with open(file) as fp:
+            fp.seek(0, 2)
+            while True:
+                readline = fp.readline()
+                line = readline.strip()
+                if line:
+                    yield f'data: {line}\n\n'
+                else:
+                    time.sleep(0.1)
 
-    def _profile_stream():
-        while True:
-            try:
-                gevent.sleep(0)
-                message = __profile_queue__.get()
-                yield f'data: {message.msg}\n\n'
-            except EOFError:
-                yield 'PROFILE ENDS\n\n'
-                break
-
-    @app.route(_config['endpoints']['log'])
+    @app.route(log_config['endpoints']['log'])
     def get_log():
         """Get the logs, endpoint `/log/stream`  """
-        return Response(_log_stream(), mimetype="text/event-stream")
+        return Response(_log_stream(log_config['files']['log']), mimetype="text/event-stream")
 
-    @app.route(_config['endpoints']['yaml'])
+    @app.route(log_config['endpoints']['yaml'])
     def get_yaml():
         """Get the yaml of the flow  """
         return flow_yaml
 
-    @app.route(_config['endpoints']['profile'])
+    @app.route(log_config['endpoints']['profile'])
     def get_profile():
         """Get the profile logs, endpoint `/profile/stream`  """
-        return Response(_profile_stream(), mimetype='text/event-stream')
+        return Response(_log_stream(log_config['files']['profile']), mimetype='text/event-stream')
 
-    @app.route(_config['endpoints']['podapi'])
+    @app.route(log_config['endpoints']['podapi'])
     def get_podargs():
         """Get the default args of a pod"""
 
@@ -111,12 +102,12 @@ def start_sse_logger(server_config_path: str, flow_yaml: str = None):
         d = {'pod': d, 'version': __version__, 'usage': parser.format_help()}
         return jsonify(d)
 
-    @app.route(_config['endpoints']['shutdown'])
+    @app.route(log_config['endpoints']['shutdown'])
     def shutdown():
         server.stop()
         return 'Server shutting down...'
 
-    @app.route(_config['endpoints']['ready'])
+    @app.route(log_config['endpoints']['ready'])
     def is_ready():
         return Response(status=200)
 
